#include <LiquidCrystal.h>

// For letters
char * letters[] = {
  ".-",
  "-...",
  "-.-.",
  "-..",
  ".",
  "..-.",
  "--.",
  "....",
  "..", // A-I
  ".---",
  "-.-",
  ".-..",
  "--",
  "-.",
  "---",
  ".--.",
  "--.-",
  ".-.", // J-R 
  "...",
  "-",
  "..-",
  "...-",
  ".--",
  "-..-",
  "-.--",
  "--.." // S-Z
};

// For Numbers
char * numbers[] = {
  "-----", // 0
  ".----", // 1
  "..---", // 2
  "...--", // 3
  "....-", // 4
  ".....", // 5
  "-....", // 6
  "--...", // 7
  "---..", // 8
  "----."  // 9
};

// Time equivalent to one dot
int dotDelay = 200;
// Pin connected to ST_CP of 74HC595
int latchPin = 13;
// Pin connected to SH_CP of 74HC595
int clockPin = 12;
// Pin connected to DS of 74HC595
int dataPin = 11;
// Pin connected to the Piezo buzzer
int tonePin = 10;
// Initial column to use for display on the LCD
int column = 0;
// Initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd0(7, 6, 5, 4, 3, 2);
LiquidCrystal lcd1(A0, A1, A2, A3, A4, A5);
// Button pins and initial state
const int buttonPin0 = 8;
int buttonState0 = 0;
int prevstate0 = 0;
const int buttonPin1 = 9;
int buttonState1 = 0;
int prevstate1 = 0;
// Interval during which changes in options are read
unsigned long previousMillis = millis();
const long interval = 5000;

void setup() {
  pinMode(latchPin, OUTPUT);
  pinMode(buttonPin0, INPUT);
  pinMode(buttonPin1, INPUT);
  // Set the LCD's number of columns and rows
  lcd0.begin(16, 2);
  lcd1.begin(16, 2);
  lcd1.print("LED        AUDIO");
  lcd1.setCursor(0, 1);
  lcd1.print("OFF          OFF");
  Serial.setTimeout(5000);
  Serial.begin(9600);
}

void loop() {
  buttonState0 = digitalRead(buttonPin0);
  toggleLEDOption(buttonState0);
  buttonState1 = digitalRead(buttonPin1);
  toggleAudOption(buttonState1);
  
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    Serial.println("Ingresa el mensaje para codificar en codigo Morse.");
    String msg = Serial.readString();
    // If nothing was received via Serial set the default message
    if (msg == "") {
      msg = "NO MSG";
    }

    lcd0.print(msg);
    for(int i = 0; i < msg.length(); i++) {
      bool isLastCh = (i + 1) == msg.length();
      char ch = msg[i];
      encodeAndOutputChar(ch, isLastCh);
    }

    delay(10000);
    column = 0;
    lcd0.clear();
    previousMillis = millis();
  }
}

void encodeAndOutputChar(char ch, bool isLastCh) {
  if (ch >= 'a' && ch <= 'z') {
    outputSequence(letters[ch - 'a'], isLastCh);
  } else if (ch >= 'A' && ch <= 'Z') {
    outputSequence(letters[ch - 'A'], isLastCh);
  } else if (ch >= '0' && ch <= '9') {
    outputSequence(numbers[ch - '0'], isLastCh);
  } else if (ch == ' ') {
    writeToLCD(' ', 1);
    delay(dotDelay * 4);
  }
}

void outputSequence(char * sequence, bool isLastCh) {
  int i = 0;
  while (sequence[i] != NULL) {
    bool lastPart = sequence[i + 1] == NULL;
    outputDotOrDash(sequence[i], lastPart);
    i++;
  }

  if (!isLastCh) {
    delay(dotDelay * 3);
  }
}

void outputDotOrDash(char dotOrDash, bool lastPart) {
  writeToLCD(dotOrDash, 1);

  int delayx;
  if (dotOrDash == '.') {
    delayx = dotDelay;
  } else { // must be a - 
    delayx = dotDelay * 3;
  }
  if (prevstate1 == 1) {
    buzzerOut(delayx);
  }
  if (prevstate0 == 1) {
    blinkAll_2Bytes(1, delayx);
  }

  if (!lastPart) {
    delay(dotDelay);
  }
}

void writeToLCD(char dotOrDash, int times) {
  for (int i = 0; i < times; i++) {
    lcd0.setCursor(column, 1);
    lcd0.print(dotOrDash);
    column++;
    if (column > 15) {
      column = 0;
    }
  }
}

// This shifts 8 bits out MSB first, 
// on the rising edge of the clock,
// clock idles low.
void shiftOut(int myDataPin, int myClockPin, byte myDataOut) {
  //internal function setup
  int i=0;
  int pinState;
  pinMode(myClockPin, OUTPUT);
  pinMode(myDataPin, OUTPUT);

  //clear everything out just in case to
  //prepare shift register for bit shifting
  digitalWrite(myDataPin, 0);
  digitalWrite(myClockPin, 0);

  //for each bit in the byte myDataOut
  //NOTICE THAT WE ARE COUNTING DOWN in our for loop
  //This means that %00000001 or "1" will go through such
  //that it will be pin Q0 that lights. 
  for (i=7; i>=0; i--)  {
    digitalWrite(myClockPin, 0);

    //if the value passed to myDataOut and a bitmask result 
    // true then... so if we are at i=6 and our value is
    // %11010100 it would the code compares it to %01000000 
    // and proceeds to set pinState to 1.
    if ( myDataOut & (1<<i) ) {
      pinState= 1;
    }
    else {	
      pinState= 0;
    }

    //Sets the pin to HIGH or LOW depending on pinState
    digitalWrite(myDataPin, pinState);
    //register shifts bits on upstroke of clock pin  
    digitalWrite(myClockPin, 1);
    //zero the data pin after shift to prevent bleed through
    digitalWrite(myDataPin, 0);
  }

  //stop shifting
  digitalWrite(myClockPin, 0);
}

// Blinks the whole register based on the number of times you want to 
// blink "n" and the pause between them "d"
// starts with a moment of darkness to make sure the first blink
// has its full visual effect.
void blinkAll_2Bytes(int n, int d) {
  digitalWrite(latchPin, 0);
  shiftOut(dataPin, clockPin, 0);
  shiftOut(dataPin, clockPin, 0);
  digitalWrite(latchPin, 1);
  delay(200);
  for (int x = 0; x < n; x++) {
    digitalWrite(latchPin, 0);
    shiftOut(dataPin, clockPin, 255);
    shiftOut(dataPin, clockPin, 255);
    digitalWrite(latchPin, 1);
    delay(d);
    digitalWrite(latchPin, 0);
    shiftOut(dataPin, clockPin, 0);
    shiftOut(dataPin, clockPin, 0);
    digitalWrite(latchPin, 1);
    delay(d);
  }
}

void buzzerOut(int duration) {
  tone(tonePin, 262, duration);
  // to distinguish the notes, set a minimum time between them
  delay(duration * 1.3);  
}

void toggleLEDOption(int buttonState) {
  if (buttonState == 1 && prevstate0 == 0) {
    lcd1.setCursor(0, 1);
    lcd1.print("ON ");
    prevstate0++;
  } else if (buttonState == 1 && prevstate0 == 1) {
    lcd1.setCursor(0, 1);
    lcd1.print("OFF");
    prevstate0--;
  }
  delay(500);
}

void toggleAudOption(int buttonState) {
  if (buttonState == 1 && prevstate1 == 0) {
    lcd1.setCursor(13, 1);
    lcd1.print("ON ");
    prevstate1++;
  } else if (buttonState == 1 && prevstate1 == 1) {
    lcd1.setCursor(13, 1);
    lcd1.print("OFF");
    prevstate1--;
  }
  delay(500);
}
